!+
! Module: eppatplot_mod
!
! Authors:  Michael Freyberg - MPE Garching
!           Frank Haberl - MPE Garching
!           Konrad Dennerl - MPE Garching
!          
! Description: Plot EPIC pn event pattern statistics
!
! Comments:
!
!-

module eppatplot_mod

  use dal
  use param
  use errorHandling

  implicit none

  private

  public eppatplot

contains

!---------------------------------------------------------------------
subroutine eppatplot
!---------------------------------------------------------------------

  implicit none

  character(len=10)  ,parameter  :: CodeVersion = '1.0.2'
  integer(kind=Int32),parameter  :: maxbin = 4096
  integer(kind=Int32),parameter  :: maxpat = 4
! integer             :: statusPG, pgopen
!
  logical(kind=bool)             :: enable_qdp = .false.
!
  real(kind=single)   ::  h, he, hs, hd, ht, hq, sigma, s,        &
                    xmin, xmax, ymin, ymax, xtxt, ytxt,           &
                    xdat, ydat, xsiz, ysiz,                       &
                    vxmin, vxmax, vymin, vymax, grey,             &
                    sprb(maxbin), rhis(maxpat,maxbin),            &
                    xhis(maxpat,2*maxbin), yhis(maxpat,2*maxbin), &
                    model(2,maxbin), livetime, y
  integer(kind=Int32) ::  ntrow, i, j, k,                         &
                    kbin, ibin, ks, kd, kt, kq, kpat,             &
                    khis(maxpat+1,maxbin), jm(maxbin),            &
                    jp(maxbin), ipat, jpat, jcts(maxbin),         &
                    mbin, icol(maxpat), k1, k2, nbin(maxpat)
  integer             :: iw, ierr, dthresh, calMode
  logical(kind=bool)  :: is_ps = .false., is_gif = .false., flag0 = .true.
  logical(kind=bool)  :: xaxisadu = .false., useplotfile = .false., &
                         withflag = .true.
! character(len=1)    :: chpt(maxpat)
  character(len=2)    :: ccdmode
  character(len=40)   :: instmode, device, ext, xlabel, instrument, insttxt
  character(len=1024) :: infilename,outfilename,canonicalname,outdir,plotfile
  character(len=1024) :: outqdpname
!
  type(DataSetT)      :: infile
  type(TableT)        :: events
  type(ColumnT)       :: pattern,pi,flag
  integer(kind=Int8),  dimension(:), pointer :: patternCol
  integer(kind=Int16), dimension(:), pointer :: piCol
  integer(kind=Int32), dimension(:), pointer :: flagCol
!
  real(kind=single)           ::  conv
! real(kind=single),parameter ::  conv = 5.e-3  ! keV/adu
! real(kind=single),parameter ::  conv = 1.0    ! adu channel
  real(kind=single),parameter ::  econv = 5.0   ! eV per adu channel
!
  data icol   / 2, 4, 3, 5 /
! data chpt   / 's', 'd', 't', 'q' /
!
!-----------------------------------------------------------------------
! start the program:
! -------------------------------------------
!  
  write(errstr,*) 'EPPATPLOT ',trim(CodeVersion),' is running...'
  call message(AppMsg,Sparse)

! -------------------------------------------
! initialize the histograms:
! -------------------------------------------

  khis = 0

! -------------------------------------------
! get input parameters:
! -------------------------------------------

  call getInParamValues(infilename, sigma, device, outdir, withflag, &
                            useplotfile, plotfile, xaxisadu, enable_qdp)
  call getCanonicalName(infilename, canonicalname)

  select case (device)
    case ('/VCPS')
      ext = 'ps'
    case ('/GIF')
      ext = 'gif'
    case default
      ext = 'ps'
  end select

  if (useplotfile) then
    outfilename = trim(outdir) // '/' // trim(plotfile)
    outqdpname  = trim(outdir) // '/' // trim(canonicalname) // '.qdp'
  else
    outfilename = trim(outdir) // '/' // trim(canonicalname) // '.' // trim(ext)
    outqdpname  = trim(outdir) // '/' // trim(canonicalname) // '.qdp'
  endif
! print*,'|',trim(outfilename),'|'

  if (xaxisadu) then
    conv = 1.0
    xlabel = 'ADU Channel [5 eV]'
  else
    conv = 5.e-3
    conv = 5.0
    xlabel = 'PI Channel [eV]'
  endif

  write(errstr,'(1x,a,a)') 'Accessing input data set: ',trim(infilename)
  call message(AppMsg,Verbose)

  if (.not.setexists(infilename)) then
    call error("noSet","Input data set not found")
  endif
  infile = dataSet(infilename,READ,LOW_MEMORY)

  if (.not. hasBlock(infile,'EVENTS')) then    ! no EVENTS extension
    call error("noEvents","No EVENTS extension in input data set")
  endif
  events = table(infile,'EVENTS',READ)
!
! read INSTRUME attribute and fill relevant text string
!
  if (hasAttribute(events,"INSTRUME")) then
    instrument = trim(stringAttribute(events,"INSTRUME"))
    write(errstr,'(1x,a,t39,a)') 'Instrument: ',trim(instrument)
    call message(AppMsg,Verbose)
  else
    instrument = ' '
    call message(AppMsg,Verbose,'INSTRUME attribute not found')
  endif

  select case (instrument)
    case ('EPN')
      insttxt = 'EPIC pn'
    case ('EMOS1')
      insttxt = 'EPIC MOS1'
    case ('EMOS2')
      insttxt = 'EPIC MOS2'
    case ('RGS1')
      insttxt = 'RGS1'
    case ('RGS2')
      insttxt = 'RGS2'
    case default
      insttxt = 'EPIC pn'
  end select

  ntrow = numberOfRows(events)
  if (ntrow .le. 0) then
    call error('emptyInput','No events in input file.')
  else
    write(errstr,'(1x,a,t35,i9)') 'Number of events in input file: ',ntrow
    call message(AppMsg,Verbose)    
  endif

  if (.not.hasColumn(events,'PATTERN')) then
    call error("noPattern","No PATTERN column in input data set")
  endif
  pattern = column(events,'PATTERN',READ)
  patternCol => int8data(pattern)

  if (.not.hasColumn(events,'PI')) then
    call error("noPi","No PI column in input data set")
  endif
  pi = column(events,'PI',READ)
  piCol => int16data(pi)

  if (.not.hasColumn(events,'FLAG')) then
    if (withflag) then
      call warning("noFlag", &
          "No FLAG column in input data set, using all events for plots.")
      flag0 = .true.
      withflag = .false.
    endif
  endif
  flag = column(events,'FLAG',READ)
  flagCol => int32data(flag)

  if (hasAttribute(events,"SUBMODE")) then
    instmode = trim(stringAttribute(events,"SUBMODE"))
  else
    instmode = ' '
    call message(AppMsg,Verbose,'SUBMODE attribute not found')
  endif

  if (hasAttribute(events,"LIVETIME")) then
    livetime = real64Attribute(events,"LIVETIME")
    write(errstr,'(1x,a,t35,f9.1,a)') 'Livetime by evselect: ',livetime,' [s]'
    call message(AppMsg,Verbose)
  else
    livetime = 0.0
  endif

  select case (instmode)
    case ('PrimeFullWindow')
      ccdmode = 'FF'
      calMode = 0
      dthresh = 40
    case ('PrimeFullWindowExtended')
      ccdmode = 'FF'
      calMode = 1
      dthresh = 40
    case ('PrimeLargeWindow')
      ccdmode = 'SW'
      calMode = 2
      dthresh = 40
    case ('PrimeSmallWindow')
      ccdmode = 'SW'
      calMode = 3
      dthresh = 40
    case ('FastTiming')
      ccdmode = '  '
      calMode = 4
      dthresh = 80
    case ('FastBurst')
      ccdmode = '  '
      calMode = 5
      dthresh = 46
    case default
      ccdmode = '  '
      calMode = 0
      dthresh = 40
  end select
  write(errstr,'(1x,a,t35,a)') 'Instrument mode of observation:',trim(instmode)
  call message(AppMsg,Verbose)
  write(errstr,'(1x,a,t35,f9.1)') 'Sigma for rebinning: ',sigma
  call message(AppMsg,Verbose)


! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  call message(AppMsg,Noisy,'Reading input data set ...')

  do i = 1, ntrow
    jpat = patternCol(i-1)
    if (withflag) flag0 = (flagCol(i-1) == 0)
    if (jpat.eq.0 .and. flag0) then                       ! singles
      kpat = 1
    elseif (jpat.ge.1.and.jpat.le.4 .and. flag0) then     ! doubles
      kpat = 2
    elseif (jpat.ge.5.and.jpat.le.8 .and. flag0) then     ! triples
      kpat = 3
    elseif (jpat.ge.9.and.jpat.le.12 .and. flag0) then    ! quadruples
      kpat = 4
    else
      kpat = 5
    endif

    kbin = piCol(i-1)/econv

    if (kpat.ge.1 .and. kpat.le.maxpat .and.   &
        kbin.ge.1 .and. kbin.le.maxbin) then
      khis(kpat,kbin) = khis(kpat,kbin) + 1
    endif
  enddo
!
  if (withflag) call release(flag)
  call release(pattern)
  call release(pi)
  call release(events)
  call release(infile)
!
!-----------------------------------------------------------------------
!
!     accumulate the distributions
!
  call message(AppMsg,Sparse,' ')
  call message(AppMsg,Verbose,' Pattern statistics for recognized events:')

  ks = 0
  kd = 0
  kt = 0
  kq = 0
  ymax = 0.0
  ymin = 1.0e+6
!
  do ibin = 1, maxbin
    ks = ks + khis(1,ibin)
    kd = kd + khis(2,ibin)
    kt = kt + khis(3,ibin)
    kq = kq + khis(4,ibin)
    khis(5,ibin) = khis(1,ibin) + khis(2,ibin) + khis(3,ibin) + khis(4,ibin)
  enddo
!
  k = ks + kd + kt + kq
!
  hs = real(ks) / real(k)
  hd = real(kd) / real(k)
  ht = real(kt) / real(k)
  hq = real(kq) / real(k)
!
  write(errstr,'(4(7x,a1))') 's','d','t','q'
  call message(AppMsg,Sparse)
  write(errstr,'(4f8.4)') hs,hd,ht,hq
  call message(AppMsg,Sparse)

!
! compute the pattern distributions vs. channel:
!
  call message(AppMsg,Noisy,' ... compute pattern distributions ...')
  do ipat = 1, maxpat
    h = sigma * sigma
    s = 0.0
    i = 1
    j = 0
    k = 0
    do while (khis(ipat,i).eq.0 .and. i.lt.maxbin)
      i = i + 1
    enddo
    jm(1) = i
    if (i.gt.1) i = i - 1
    do while (i.lt.maxbin)
      i = i + 1
      s = s + real(khis(ipat,i))
      if (s*s.gt.h*s) then
        k = k + 1
        jcts(k) = s
        jp(k) = i
        s = 0.0
      endif
    enddo
    mbin = k

    if (mbin > 0) then
      do i = 2, mbin
        jm(i) = jp(i-1) + 1
      enddo

      do i = 1, mbin
        h = real(jp(i)-jm(i)) + 1.0
        sprb(i)  = real(jcts(i)) / h
      enddo
!
! store as histogram
!
      j = j+1
      xhis(ipat,j) = alog10((real(jm(1))-0.5)*conv)
      y = alog10(sprb(1))
      yhis(ipat,j) = y
      j = j+1
      xhis(ipat,j) = alog10((real(jp(1))+0.5)*conv)
      yhis(ipat,j) = y
      ymax = max(ymax,y)
      ymin = min(ymin,y)
      do i = 2, mbin
        j = j+1
        xhis(ipat,j) = xhis(ipat,j-1) 
        y = alog10(sprb(i))
        yhis(ipat,j) = y
        j = j+1
        xhis(ipat,j) = alog10((real(jp(i))+0.5)*conv)
        yhis(ipat,j) = y
        ymax = max(ymax,y)
        ymin = min(ymin,y)
      enddo
    else
      write(errstr,'(a,i1)') &
          'not enough statistics for at least 1 bin for PAT = ',ipat
      call warning('sigmaTooLarge')
    endif
    nbin(ipat) = j
  enddo

!
! initialize the plot for pattern distributions:
!
  call message(AppMsg,Noisy,' ')
  call message(AppMsg,Noisy,' ... prepare plot input data set ...')

  if (index(trim(device),"PS") .gt. 0)  is_ps = .true.
  if (index(trim(device),"GIF") .gt. 0) is_gif = .true.

! statusPG = pgopen(trim(device))
  if (is_ps) then
    call pgbegin(0,trim(outfilename)//trim(device),1,1)
  elseif (is_gif) then
    call pgbegin(0,trim(outfilename)//trim(device),1,1)
  else
    call pgbegin(0,trim(device),1,1)
  endif
  call pgpaper(7.8,1.414)
!
! top bar with some text
!
  vxmin = 0.15 - 0.75*0.12
  vxmax = 0.90
  vymin = 0.98
  vymax = 1.00
  grey  = 0.95
  call pgscr(255,grey,grey,grey)
  call pgvport(vxmin,vxmax,vymin,vymax)
  call pgwindow(0.0,1.0,0.0,1.0)
  call pgsci(255)
  call pgrect(0.0,1.0,0.0,1.0)
  call pgsci(1)
  call pgsch(0.5)
  call pgslw(1)
  call pgptext(0.99, 0.4, 0.0, 1.0, trim(outfilename))
  call pgptext(0.01, 0.4, 0.0, 0.0, & 
           'XMM '//trim(insttxt)// &
           ', pattern statistics: eppatplot-'//trim(CodeVersion))
!
! username, date and time at bottom of plot
!
  call pgsch(1.0)
  call pgiden

  xmin = alog10(17.0)
  xmax = alog10(4096.0)
!  ymin = max(alog10(1.e-3),real(int(alog10(ymin)-1)))
!  ymax = real(int(alog10(ymax)+1))
  ymin = int(max(-3.0,ymin))-1
  ymax = int(ymax)+1

  xsiz = xmax - xmin
  ysiz = ymax - ymin
  vxmin = 0.15
  vxmax = 0.90
  vymin = 0.52
  vymax = 0.90
  call pgvport(vxmin,vxmax,vymin,vymax)
  call pgwindow(xmin,xmax,ymin,ymax)
  call pgsci(255)
  call pgrect(xmin,xmax,ymin,ymax)
  call pgsci(1)
  call pgslw(3)
  call pgbox('BC',0.0,0,'BC',0.0,0)
  call pgbox('LN',0.0,0,' ',0.0,0)
  call pgsch(0.75)
  call pgbox(' ',0.0,0,'LNV',0.0,0)
  call pgsch(1.0)
  call pgslw(1)
  call pgbox('BCLST',0.0,0,'BCLST',0.0,0)
  call pgslw(5)
  call pgsch(1.1)
  xtxt = (xmax + xmin)*0.5
  ytxt = ymin - ysiz*0.11
  call pgptext(xtxt, ytxt, 0.0, 0.5, trim(xlabel))
  call pgsch(1.25)
  call pgslw(4)
  xtxt = xmin - xsiz*0.12
  ytxt = ymax + ysiz*0.12
  call pgptext(xtxt, ytxt, 0.0, 0.0, 'XMM / '//trim(insttxt))
  call pgsch(1.0)
  call pgslw(3)
  call pgsch(0.75)
  ytxt = ymax + ysiz*0.04
  xtxt = xmax
  call pgptext(xtxt, ytxt, 0.0, 1.0, trim(infilename))
  call pgsch(1.0)
  call pgslw(4)
  call pgsci(5)
  xtxt = xmin - xsiz*0.1
  ytxt = ymin + ysiz*0.125
  call pgptext(xtxt, ytxt, 90.0, 0.5, 'q/adu')
  call pgsci(3)
  ytxt = ytxt + ysiz*0.25
  call pgptext(xtxt, ytxt, 90.0, 0.5, 't/adu')
  call pgsci(4)
  ytxt = ytxt + ysiz*0.25
  call pgptext(xtxt, ytxt, 90.0, 0.5, 'd/adu')
  call pgsci(2)
  ytxt = ytxt + ysiz*0.25
  call pgptext(xtxt, ytxt, 90.0, 0.5, 's/adu')
  call pgsci(1)
  call pgslw(1)
!
! plot the pattern distributions vs. channel:
!
  call message(AppMsg,Noisy,' ... plot pattern distributions ...')
  do ipat = 1, maxpat
    j = 0
!
! plot as histogram
!
    if (nbin(ipat) .gt. 0) then
      call pgsci(icol(ipat))
      j = j+1
      xdat = xhis(ipat,j)
      ydat = yhis(ipat,j)
      call pgmove(xdat,ydat)
      j = j+1
      xdat = xhis(ipat,j)
      call pgdraw(xdat,ydat)
      do i = j+1, nbin(ipat), 2
        ydat = yhis(ipat,i)
        call pgdraw(xdat,ydat)
        xdat = xhis(ipat,i+1)
        call pgdraw(xdat,ydat)
      enddo
    endif
  enddo
!
! initialize the plot for pattern statistics:
!
  call message(AppMsg,Noisy,' ... plot pattern statistics ...')
  ymin = -0.05
  ymax = 1.05
  ysiz = ymax - ymin
  vymin = vymin - 0.45
  vymax = vymax - 0.45
  call pgvport(vxmin,vxmax,vymin,vymax)
  call pgwindow(xmin,xmax,ymin,ymax)
  call pgsci(255)
  call pgrect(xmin,xmax,ymin,ymax)
  call pgsci(1)
  call pgslw(3)
  call pgbox('BC',0.0,0,'BC',0.0,0)
  call pgbox('LN',0.0,0,' ',0.0,0)
  call pgsch(0.75)
  call pgbox(' ',0.0,0,'NV',0.0,0)
  call pgsch(1.0)
  call pgslw(1)
  call pgbox('A',0.0,0,' ',0.0,0)
  call pgbox('BCLST',0.0,0,'BCST',0.0,0)
! call pgslw(6)
  call pgslw(5)
  call pgsch(1.1)
  xtxt = (xmax + xmin)*0.5
  ytxt = ymin - ysiz*0.11
  call pgptext(xtxt, ytxt, 0.0, 0.5, trim(xlabel))
  xtxt = xmin + xsiz*0.75
  ytxt = ymin + ysiz*0.80
  call pgptext(xtxt, ytxt, 0.0, 0.5, ccdmode//' Model')
  call pgslw(4)
  call pgsci(icol(4))
  xtxt = xmin - xsiz*0.1
  ytxt = ymin + ysiz*0.125
  call pgptext(xtxt, ytxt, 90.0, 0.5, 'q')
  call pgsci(icol(3))
  ytxt = ytxt + ysiz*0.25
  call pgptext(xtxt, ytxt, 90.0, 0.5, 't')
  call pgsci(icol(2))
  ytxt = ytxt + ysiz*0.25
  call pgptext(xtxt, ytxt, 90.0, 0.5, 'd')
  call pgsci(icol(1))
  ytxt = ytxt + ysiz*0.25
  call pgptext(xtxt, ytxt, 90.0, 0.5, 's')
  call pgsci(1)
  call pgslw(1)
!
! determine binning for statistics plot
!
  call message(AppMsg,Noisy,' ... determine binning for statistics plot ...')
  ipat = 5
  h = sigma * sigma * 10.0  * 10.0
  s = 0.0
  i = 1
  k = 0
  do while (khis(ipat,i).eq.0 .and. i.lt.maxbin)
    i = i + 1
  enddo
  jm(1) = i
  if (i.gt.1) i = i - 1
  do while (i.lt.maxbin)
    i = i + 1
    s = s + real(khis(ipat,i))
    if (s*s.gt.h*s) then
      k = k + 1
      jp(k) = i
      s = 0.0
    endif
  enddo
  mbin = k
  do i = 2, mbin
    jm(i) = jp(i-1) + 1
  enddo
!
! determine the fraction of patterns among the recognized photons:
!
  call message(AppMsg,Noisy,&
    ' ... determine the fraction of patterns among the recognized photons ...')
  do ipat = 1, maxpat
    do i = 1, mbin
      k1 = 0
      k2 = 0
      do ibin = jm(i), jp(i)
        k1 = k1 + khis(ipat,ibin)
        k2 = k2 + khis(5,ibin)
      enddo
      rhis(ipat,i) = real(k1) / amax1(real(k2),1.0)
    enddo
!
! plot as histogram
!
    call pgsci(icol(ipat))
    xdat = alog10((real(jm(1))-0.5)*conv)
    ydat = rhis(ipat,1)
    call pgmove(xdat,ydat)
    xdat = alog10((real(jp(1))+0.5)*conv)
    call pgdraw(xdat,ydat)
    do i = 2, mbin
      ydat = rhis(ipat,i)
      call pgdraw(xdat,ydat)
      xdat = alog10((real(jp(i))+0.5)*conv)
      call pgdraw(xdat,ydat)
    enddo
  enddo
!------------------------------------------------------------------------------
! plot the standard single and double pattern statistics 
! as used in the response matrix
!
  call message(AppMsg,Noisy,' ... model curve as used for DRM ...')
  do ipat = 1,2
    model(ipat,1:maxbin) = 0.0
    call patmod(model(ipat,1:maxbin), maxbin, calMode, dthresh, ipat)
  enddo
!
! plot as line
!
  call message(AppMsg,Noisy,' ... plot as line ...')
  do ipat = 1,2
    call pgsci(icol(ipat))
    i = 20
    xdat = alog10(real(i))
    ydat = model(ipat,i)
    call pgmove(xdat,ydat)
    do i = 21, maxbin
       xdat = alog10(real(i))
       ydat = model(ipat,i)
       call pgdraw(xdat,ydat)
    enddo
  enddo
  call pgend
!
! write out results into ASCII file which can be plotted with QDP
!
  if (enable_qdp) then
    call get_io_Unit(iw)
    open(iw,file=outqdpname,status='unknown',iostat=ierr)
    if (ierr .eq. 0) then
      write(iw,'(a,a)') '! ',trim(outqdpname)
      write(iw,'(a)')'read serr 1'
      write(iw,'(a)')'col 0 on 2 3'
      write(iw,'(a)')'label x ADU Channel'
      write(iw,'(a)')'label y Pattern fraction'
      write(iw,'(a)')'resc y -0.05 1.05'
      write(iw,'(a)')'log x'
      write(iw,'(a)') '! CH           CHmin  CHmax  s'//&
                                  '        d        t        q'
      write(iw,'(a)') '! -----        -----  -----  ------   '//&
                                    '------   ------   ------'
  !
      do i = 1, mbin
        h = (jp(i)+jm(i))*0.5
        he = (jp(i)-jm(i))*0.5
        write(iw,'(2f6.1,2i7,4f9.4)') h,he,jm(i),jp(i),&
            rhis(1,i),rhis(2,i),rhis(3,i),rhis(4,i)
      enddo
      close(iw)
      call message(AppMsg,Verbose,' ')
      write(errstr,*) 'QDP output written to file ' // trim(outqdpname)
    else
      write(errstr,*) 'QDP output file could not be opened:' // trim(outqdpname)
    endif
    call message(AppMsg,Verbose)
  else
    call message(AppMsg,Verbose,' ')
  endif

  write(errstr,*) 'PostScript output written to file ' // trim(outfilename)
  call message(AppMsg,Verbose)
  call message(AppMsg,Verbose,' ')

end subroutine eppatplot


! =========================
! Program parameter reading
! =========================
subroutine getInParamValues(infilename, sigma, device, outdir, withflag, &
                            useplotfile, plotfile, xaxisadu, enable_qdp)

  real(kind=single)  ,intent(out)   :: sigma
  logical(kind=bool) ,intent(out)   :: useplotfile,xaxisadu,enable_qdp,withflag
  character(len=*)   ,intent(out)   :: infilename,device,outdir,plotfile

  infilename  = trim(stringParameter('set'))
  sigma       = realParameter('sigma')
  withflag    = booleanParameter('withflag')
  useplotfile = booleanParameter('useplotfile')
  device      = trim(stringParameter('device'))
  outdir      = trim(stringParameter('outdir'))
  plotfile    = trim(stringParameter('plotfile'))
  xaxisadu    = booleanParameter('xaxisadu')
  enable_qdp  = booleanParameter('withqdp')

  if (.NOT. xaxisadu) then
    call message(AppMsg,Sparse,'Parameter setting XAXISADU=N not ' // &
                'implemented yet, using XAXISADU=Y in the following.')
    xaxisadu = .true.
  endif

end subroutine getInParamValues


! =========================
! Program parameter reading
! =========================
subroutine getCanonicalName(infilename,canonicalname)

  implicit none
  character(len=*)   ,intent(in)    :: infilename
  character(len=*)   ,intent(out)   :: canonicalname
  integer                           :: ns,nd,nl,il

  nl = len(trim(infilename))
  do il = nl, 1, -1
    if (infilename(il:il) == '/') exit
  enddo
  ns = max(il,1)
! print*,'IL,NS = ',il,ns
  do il = nl, ns, -1
    if (infilename(il:il) == '.') exit
  enddo
  if (il <= ns) then
    nd = nl
  else
    nd = max(il,2)-1
  endif
! print*,'IL,ND = ',il,nD
  canonicalname = infilename(ns:nd) // '_pat'
! print*,'|',trim(canonicalname),'|'

end subroutine getCanonicalName


! ==========================================================================
! suggested by Clive Page in SSC-LUX-TN-0005 (Issue 1.0, 1996 Sept 2)
! ==========================================================================
subroutine get_io_unit(iounit)

  implicit none
  integer, intent(out) :: iounit
  logical              :: open, exists
  do iounit = 99,0,-1
    inquire(unit=iounit, exist=exists, opened=open)
    if (exists .and. .not. open) exit
  enddo
  
end subroutine get_io_unit


! ==========================================================================
! returns the pattern distribution model
! ==========================================================================
subroutine patmod(patfraction, maxbin, calMode, dthresh, pattyp)
!
  implicit none
  integer          ,intent(in)    :: calMode, dthresh, pattyp, maxbin
  real(kind=single),intent(out)   :: patfraction(1:maxbin)
  integer                         :: ibin
  real(kind=single)               :: rbin
! character(len=*)   ,intent(in)  :: ccdmode
!
! single-event pattern
!
  if (pattyp.eq.1) then
!   if (ccdmode.eq.'FF') then
    if (calMode == 0) then
      patfraction(1:dthresh-1) = 1.0
      do ibin = dthresh, maxbin
        rbin = 2.0*real(ibin-dthresh)
        patfraction(ibin) = min(1.0,  &
                 0.5331+0.1544/(1.0+(rbin/1329.)**2)  &
                       +0.2007/(1.0+(rbin/180.6)**2)  &
                       +0.1181/(1.0+(rbin/34.89)**2))
      enddo
!   elseif (ccdmode.eq.'LW') then
    elseif (calMode == 2) then
      patfraction(1:dthresh-1) = 1.0
      do ibin = dthresh, maxbin
        rbin = 2.0*real(ibin-dthresh)
! parameters derived with CTE version 1 May 2001
         patfraction(ibin) = min(1.0,  &
                0.4913+0.1857/(1.0+(rbin/1613.)**2)  &
                      +0.2012/(1.0+(rbin/175.7)**2)  &
                      +0.1223/(1.0+(rbin/30.95)**2))
       enddo
!   elseif (ccdmode.eq.'SW') then
    elseif (calMode == 3) then
      patfraction(1:dthresh-1) = 1.0
      do ibin = dthresh, maxbin
        rbin = 2.0*real(ibin-dthresh)
!
! parameters derived with CTE version Jan 2001
!
!         patfraction(ibin) = min(1.0,  &
!    &           0.5532+0.1497/(1.0+(rbin/943.1)**2)  &
!    &                 +0.1829/(1.0+(rbin/136.9)**2)  &
!    &                 +0.1178/(1.0+(rbin/33.42)**2))
!
! parameters derived with CTE version end Mar 2001
!
!         patfraction(ibin) = min(1.0,  &
!    &           0.5425+0.1873/(1.0+(rbin/927.6)**2)  &
!    &                 +0.1751/(1.0+(rbin/100.8)**2)  &
!    &                 +0.09186/(1.0+(rbin/15.82)**2))
!
! parameters derived with CTE version 11.04.2001
!
        patfraction(ibin) = min(1.0,  &
                0.5393+0.1686/(1.0+(rbin/1028.)**2)  &
                      +0.1883/(1.0+(rbin/124.2)**2)  &
                      +0.1039/(1.0+(rbin/23.82)**2))
      enddo
!   elseif (ccdmode.eq.'TI') then
    elseif (calMode == 4) then
      patfraction(1:dthresh-1) = 1.0
      do ibin = dthresh, maxbin
         rbin = 2.0*real(ibin-dthresh)
! parameters derived with CTE version Apr 2001
         patfraction(ibin) = min(1.0,  &
                0.5858+0.2213/(1.0+(rbin/7812.)**2)  &
                      +0.0640/(1.0+(rbin/1311.)**2)  &
                      +0.1252/(1.0+(rbin/203.0)**2))
      enddo
    else
      patfraction(1:maxbin) = 0.
    endif
!
! double-event pattern
!
  elseif (pattyp.eq.2) then
!   if (ccdmode.eq.'FF') then
    if (calMode == 0) then
      patfraction(1:dthresh-1) = 0.0
      do ibin = dthresh, maxbin
         rbin = 2.0*real(ibin-dthresh)
         patfraction(ibin) = max(0.0,  &
                0.3988-0.1016/(1.0+(rbin/1359.)**2)  &
                      -0.1836/(1.0+(rbin/176.2)**2)  &
                      -0.1198/(1.0+(rbin/35.23)**2))
      enddo
!   elseif (ccdmode.eq.'LW') then
    elseif (calMode == 2) then
      patfraction(1:dthresh-1) = 0.0
      do ibin = dthresh, maxbin
         rbin = 2.0*real(ibin-dthresh)
!        parameters derived with CTE version 1 May 2001
         patfraction(ibin) = max(0.0,  &
                0.4336-0.1301/(1.0+(rbin/1706.)**2)  &
                      -0.1817/(1.0+(rbin/169.6)**2)  &
                      -0.1227/(1.0+(rbin/30.54)**2))
      enddo
!   elseif (ccdmode.eq.'SW') then
    elseif (calMode == 3) then
      patfraction(1:dthresh-1) = 0.0
      do ibin = dthresh, maxbin
         rbin = 2.0*real(ibin-dthresh)
! parameters derived with CTE version Jan 2001
!        patfraction(ibin) = max(0.0,  &
!               0.3764-0.1044/(1.0+(rbin/751.6)**2)  &
!                     -0.1694/(1.0+(rbin/116.7)**2)  &
!                     -0.1066/(1.0+(rbin/31.28)**2))
! parameters derived with CTE version end Mar 2001
!        patfraction(ibin) = max(0.0,  &
!               0.3850-0.1346/(1.0+(rbin/771.6)**2)  &
!                     -0.1637/(1.0+(rbin/87.47)**2)  &
!                     -0.08439/(1.0+(rbin/14.18)**2))
! parameters derived with CTE version 11.04.2001
          patfraction(ibin) = max(0.0,  &
                0.3857-0.1154/(1.0+(rbin/848.3)**2)  &
                      -0.1719/(1.0+(rbin/110.6)**2)  &
                      -0.09855/(1.0+(rbin/22.95)**2))
      enddo
!   elseif (ccdmode.eq.'TI') then
    elseif (calMode == 4) then
      patfraction(1:dthresh-1) = 0.0
      do ibin = dthresh, maxbin
         rbin = 2.0*real(ibin-dthresh)
! parameters derived with CTE version Apr 2001
         patfraction(ibin) = max(0.0,  &
                0.2799-0.07587/(1.0+(rbin/5714.)**2)  &
                      -0.06955/(1.0+(rbin/1828.)**2)  &
                      -0.1320/(1.0+(rbin/217.1)**2))
      enddo
    else
      patfraction(1:maxbin) = 0.0
    endif
  endif

end subroutine patmod

end module eppatplot_mod
